from glob import glob
from subprocess import run
from sys import executable
from numpy import mean, std
from pandas import DataFrame


def file_list(path):
    file_list = []
    for fname in glob(path):
        file_list.append(fname)
    return file_list


def create_dataframes():
    greedy = DataFrame(columns=["dataset", "distancia", "desviacion"])
    local = DataFrame(columns=["dataset", "distancia", "desviacion"])
    return greedy, local


def process_output(results):
    distances = []
    for element in results:
        for line in element:
            if line.startswith(bytes("Total distance:", encoding="utf-8")):
                line_elements = line.split(sep=bytes(":", encoding="utf-8"))
                distances.append(float(line_elements[1]))
    return distances


def populate_dataframes(greedy, local, greedy_list, local_list, dataset):
    greedy_results = process_output(greedy_list)
    local_results = process_output(local_list)
    greedy_dict = {
        "dataset": dataset,
        "distancia": mean(greedy_results),
        "desviacion": std(greedy_results),
    }
    local_dict = {
        "dataset": dataset,
        "distancia": mean(local_results),
        "desviacion": std(local_results),
    }
    greedy = greedy.append(greedy_dict, ignore_index=True)
    local = local.append(local_dict, ignore_index=True)
    return greedy, local


def script_execution(filenames, greedy, local, iterations=3):
    script = "src/main.py"
    for dataset in filenames:
        print(f"Running on dataset {dataset}")
        greedy_list = []
        local_list = []
        for _ in range(iterations):
            greedy_cmd = run(
                [executable, script, dataset, "greedy"], capture_output=True
            ).stdout.splitlines()
            local_cmd = run(
                [executable, script, dataset, "local"], capture_output=True
            ).stdout.splitlines()
            greedy_list.append(greedy_cmd)
            local_list.append(local_cmd)
        greedy, local = populate_dataframes(
            greedy, local, greedy_list, local_list, dataset
        )
    return greedy, local


def export_results(greedy, local):
    greedy.to_excel()
    local.to_excel()


if __name__ == "__main__":
    datasets = file_list(path="data/*.txt")
    greedy, local = create_dataframes()
    populated_greedy, populated_local = script_execution(datasets, greedy, local)
